'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _AbstractRouter = require('./AbstractRouter');

var _AbstractRouter2 = _interopRequireDefault(_AbstractRouter);

var _Request = require('./Request');

var _Request2 = _interopRequireDefault(_Request);

var _Response = require('./Response');

var _Response2 = _interopRequireDefault(_Response);

var _RouteFactory = require('./RouteFactory');

var _RouteFactory2 = _interopRequireDefault(_RouteFactory);

var _Dispatcher = require('../event/Dispatcher');

var _Dispatcher2 = _interopRequireDefault(_Dispatcher);

var _PageManager = require('../page/manager/PageManager');

var _PageManager2 = _interopRequireDefault(_PageManager);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// @server-side class ServerRouter extends __VARIABLE__ {__CLEAR__}\nexports.default = ServerRouter;

/**
 * The server-side implementation of the {@codelink Router} interface.
 */
class ServerRouter extends _AbstractRouter2.default {
  static get $dependencies() {
    return [_PageManager2.default, _RouteFactory2.default, _Dispatcher2.default, _Request2.default, _Response2.default];
  }

  /**
   * Initializes the router.
   *
   * @param {PageManager} pageManager The current page manager.
   * @param {RouteFactory} factory The router factory used to create routes.
   * @param {Dispatcher} dispatcher Dispatcher fires events to app.
   * @param {Request} request The current HTTP request.
   * @param {Response} response The current HTTP response.
   */
  constructor(pageManager, factory, dispatcher, request, response) {
    super(pageManager, factory, dispatcher);

    /**
     * The current HTTP request.
     *
     * @type {Request}
     */
    this._request = request;

    /**
     * The current HTTP response.
     *
     * @type {Response}
     */
    this._response = response;
  }

  /**
   * @inheritdoc
   */
  getPath() {
    return this._extractRoutePath(this._request.getPath());
  }

  /**
   * @inheritdoc
   */
  listen() {
    return this;
  }

  /**
   * @inheritdoc
   */
  redirect(url = '/', options = {}) {
    this._response.redirect(url, options.httpStatus || 302);
  }
}
exports.default = ServerRouter;

typeof $IMA !== 'undefined' && $IMA !== null && $IMA.Loader && $IMA.Loader.register('ima/router/ServerRouter', [], function (_export, _context) {
 'use strict';
 return {
   setters: [],
   execute: function () {
     _export('default', exports.default);
   }
 };
});
